function A = accumarray (subs, val, sz)
// Create an array by accumulating the elements of a vector into the positions defined by their subscripts.
//  Calling Sequence
//  A = accumarray (subs, val, sz)
// Description
// Create an array by accumulating the elements of a vector into the
// positions defined by their subscripts.  The subscripts are defined by
// the rows of the matrix @var{subs} and the values by @var{vals}.  Each row
// of @var{subs} corresponds to one of the values in @var{vals}.
//
// The size of the matrix will be determined by the subscripts themselves.
// However, if @var{sz} is defined it determines the matrix size.  The length
// of @var{sz} must correspond to the number of columns in @var{subs}.
//
// The default action of @code{accumarray} is to sum the elements with the
// same subscripts.  This behavior can be modified by defining the @var{func}
// function.  This should be a function or function handle that accepts a 
// column vector and returns a scalar.  The result of the function should not
// depend on the order of the subscripts.
//
// The elements of the returned array that have no subscripts associated with
// them are set to zero.  Defining @var{fillval} to some other value allows
// these values to be defined.
//
// Examples
// accumarray ([1,1,1;2,1,2;2,3,2;2,1,2;2,3,2], 101:105)
//  ans(:,:,1) = [101, 0, 0; 0, 0, 0]
//   ans(:,:,2) = [0, 0, 0; 206, 0, 208]
// Authors
// Copyright (C) 2007, 2008, 2009 David Bateman
// Copyright (C) 2009, 2010 VZLU Prague

//
// This file is part of Octave.
//
// Octave is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// Octave is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Octave; see the file COPYING.  If not, see
// <http://www.gnu.org/licenses/>.

// -*- texinfo -*-
// @deftypefn  {Function File} {} accumarray (@var{subs}, @var{vals}, @var{sz}, @var{func}, @var{fillval}, @var{issparse})
// @deftypefnx {Function File} {} accumarray (@var{csubs}, @var{vals}, @dots{})
//
// Create an array by accumulating the elements of a vector into the
// positions defined by their subscripts.  The subscripts are defined by
// the rows of the matrix @var{subs} and the values by @var{vals}.  Each row
// of @var{subs} corresponds to one of the values in @var{vals}.
//
// The size of the matrix will be determined by the subscripts themselves.
// However, if @var{sz} is defined it determines the matrix size.  The length
// of @var{sz} must correspond to the number of columns in @var{subs}.
//
// The default action of @code{accumarray} is to sum the elements with the
// same subscripts.  This behavior can be modified by defining the @var{func}
// function.  This should be a function or function handle that accepts a 
// column vector and returns a scalar.  The result of the function should not
// depend on the order of the subscripts.
//
// The elements of the returned array that have no subscripts associated with
// them are set to zero.  Defining @var{fillval} to some other value allows
// these values to be defined.
//
// By default @code{accumarray} returns a full matrix.  If @var{issparse} is
// logically true, then a sparse matrix is returned instead.
//
//
// The complexity in the non-sparse case is generally O(M+N), where N is the
// number of
// subscripts and M is the maximum subscript (linearized in multi-dimensional
// case).
// If @var{func} is one of @code{@@sum} (default), @code{@@max}, @code{@@min}
// or @code{@@(x) @{x@}}, an optimized code path is used. 
// Note that for general reduction function the interpreter overhead can play a
// major part and it may be more efficient to do multiple accumarray calls and
// compute the results in a vectorized manner.
// @end deftypefn
	
[nargout,nargin]=argn(0);
  if (nargin < 2 )
    error("at least two parameters necassery!");
  end
if nargin<3
	sz=[];
end

    ncol = size (subs,2);
  

    // Linearize subscripts.
    if (ncol > 1)
      if (isempty (sz))
       
          sz = mtlb_max (subs, [], 1);

      elseif (ncol ~= length (sz))
        error ("accumarray: dimensions mismatch")
      end

      // Convert multidimensional subscripts.

      subs = sub2ind (sz, subs); // creates index cache
    elseif (~ isempty (sz) & length (sz) < 2)
      error("accumarray: needs at least 2 dimensions");
    end


    // Some built-in reductions handled efficiently.


      // The general case. Reduce values. 
      n = size (subs,1);
      if  sum(size(val))==2
        val = val(ones (1, n), 1);
      else
        val = val(:);
      end
      
      // Sort indices.
      [subs, idx] = mtlb_sort (subs);
      // Identify runs.
      jdx = find (subs(1:n-1) ~= subs(2:n));
      jdx = [jdx(:); n];
      val_ind=diff ([0; jdx]);val_tmp=val(idx);val=zeros(length(val_ind),1);
for k=1:length(val_ind)
	val(k)=sum(val_tmp(sum(val_ind(1:k-1))+1:sum(val_ind(1:k))));
end
    
      subs = subs(jdx);

      // Construct matrix of fillvals.


        A = mtlb_zeros (sz);


      // Set the reduced values.
for k=1:length(val)
	     A(subs(k)) = val(k);
end
 


endfunction
